@@ -1,4 +1,6 @@
 # mypy: allow-untyped-defs
+from __future__ import annotations
+
 import contextlib
 import multiprocessing
 import os
@@ -16,8 +18,8 @@ import pytest
 @contextlib.contextmanager
 def ignore_encoding_warning():
     with warnings.catch_warnings():
-        with contextlib.suppress(NameError):  # new in 3.10
-            warnings.simplefilter("ignore", EncodingWarning)  # type: ignore [name-defined]
+        if sys.version_info >= (3, 10):
+            warnings.simplefilter("ignore", EncodingWarning)  # noqa: F821
         yield
 
 
@@ -207,13 +209,13 @@ class CommonFSTests:
 
     @pytest.mark.parametrize(
         "fil",
-        ["*dir", "*dir", pytest.mark.skip("sys.version_info <" " (3,6)")(b"*dir")],
+        ["*dir", "*dir", pytest.mark.skip("sys.version_info < (3,6)")(b"*dir")],
     )
     def test_visit_filterfunc_is_string(self, path1, fil):
         lst = []
         for i in path1.visit(fil):
             lst.append(i.relto(path1))
-        assert len(lst), 2
+        assert len(lst), 2  # noqa: PLC1802,RUF040
         assert "sampledir" in lst
         assert "otherdir" in lst
 
@@ -551,11 +553,11 @@ def batch_make_numbered_dirs(rootdir, repeats):
     for i in range(repeats):
         dir_ = local.make_numbered_dir(prefix="repro-", rootdir=rootdir)
         file_ = dir_.join("foo")
-        file_.write_text("%s" % i, encoding="utf-8")
+        file_.write_text(f"{i}", encoding="utf-8")
         actual = int(file_.read_text(encoding="utf-8"))
-        assert (
-            actual == i
-        ), f"int(file_.read_text(encoding='utf-8')) is {actual} instead of {i}"
+        assert actual == i, (
+            f"int(file_.read_text(encoding='utf-8')) is {actual} instead of {i}"
+        )
         dir_.join(".lock").remove(ignore_errors=True)
     return True
 
@@ -563,9 +565,9 @@ def batch_make_numbered_dirs(rootdir, repeats):
 class TestLocalPath(CommonFSTests):
     def test_join_normpath(self, tmpdir):
         assert tmpdir.join(".") == tmpdir
-        p = tmpdir.join("../%s" % tmpdir.basename)
+        p = tmpdir.join(f"../{tmpdir.basename}")
         assert p == tmpdir
-        p = tmpdir.join("..//%s/" % tmpdir.basename)
+        p = tmpdir.join(f"..//{tmpdir.basename}/")
         assert p == tmpdir
 
     @skiponwin32
@@ -667,7 +669,7 @@ class TestLocalPath(CommonFSTests):
         assert p == os.path.expanduser("~")
 
     @pytest.mark.skipif(
-        not sys.platform.startswith("win32"), reason="case insensitive only on windows"
+        not sys.platform.startswith("win32"), reason="case-insensitive only on windows"
     )
     def test_eq_hash_are_case_insensitive_on_windows(self):
         a = local("/some/path")
@@ -722,7 +724,7 @@ class TestLocalPath(CommonFSTests):
 
     @pytest.mark.parametrize("bin", (False, True))
     def test_dump(self, tmpdir, bin):
-        path = tmpdir.join("dumpfile%s" % int(bin))
+        path = tmpdir.join(f"dumpfile{int(bin)}")
         try:
             d = {"answer": 42}
             path.dump(d, bin=bin)
@@ -736,7 +738,6 @@ class TestLocalPath(CommonFSTests):
 
     def test_setmtime(self):
         import tempfile
-        import time
 
         try:
             fd, name = tempfile.mkstemp()
@@ -745,6 +746,7 @@ class TestLocalPath(CommonFSTests):
             name = tempfile.mktemp()
             open(name, "w").close()
         try:
+            # Do not use _pytest.timing here, as we do not want time mocking to affect this test.
             mtime = int(time.time()) - 100
             path = local(name)
             assert path.mtime() != mtime
@@ -822,7 +824,7 @@ class TestLocalPath(CommonFSTests):
         # depending on how the paths are used), but > 4096 (which is the
         # Linux' limitation) - the behaviour of paths with names > 4096 chars
         # is undetermined
-        newfilename = "/test" * 60  # type:ignore[unreachable]
+        newfilename = "/test" * 60  # type:ignore[unreachable,unused-ignore]
         l1 = tmpdir.join(newfilename)
         l1.ensure(file=True)
         l1.write_text("foo", encoding="utf-8")
@@ -853,7 +855,7 @@ class TestLocalPath(CommonFSTests):
         assert b.fnmatch(pattern)
 
     def test_sysfind(self):
-        name = sys.platform == "win32" and "cmd" or "test"
+        name = (sys.platform == "win32" and "cmd") or "test"
         x = local.sysfind(name)
         assert x.check(file=1)
         assert local.sysfind("jaksdkasldqwe") is None
@@ -898,7 +900,7 @@ class TestExecutionOnWindows:
 class TestExecution:
     pytestmark = skiponwin32
 
-    def test_sysfind_no_permisson_ignored(self, monkeypatch, tmpdir):
+    def test_sysfind_no_permission_ignored(self, monkeypatch, tmpdir):
         noperm = tmpdir.ensure("noperm", dir=True)
         monkeypatch.setenv("PATH", str(noperm), prepend=":")
         noperm.chmod(0)
@@ -946,7 +948,7 @@ class TestExecution:
                 prefix="base.", rootdir=tmpdir, keep=2, lock_timeout=0
             )
             assert numdir.check()
-            assert numdir.basename == "base.%d" % i
+            assert numdir.basename == f"base.{i}"
             if i >= 1:
                 assert numdir.new(ext=str(i - 1)).check()
             if i >= 2:
@@ -991,7 +993,7 @@ class TestExecution:
         for i in range(10):
             numdir = local.make_numbered_dir(prefix="base2.", rootdir=tmpdir, keep=2)
             assert numdir.check()
-            assert numdir.basename == "base2.%d" % i
+            assert numdir.basename == f"base2.{i}"
             for j in range(i):
                 assert numdir.new(ext=str(j)).check()
 
@@ -1248,7 +1250,7 @@ class TestWINLocalPath:
     def test_chmod_simple_int(self, path1):
         mode = path1.stat().mode
         # Ensure that we actually change the mode to something different.
-        path1.chmod(mode == 0 and 1 or 0)
+        path1.chmod((mode == 0 and 1) or 0)
         try:
             print(path1.stat().mode)
             print(mode)
@@ -1368,8 +1370,8 @@ class TestPOSIXLocalPath:
         assert realpath.basename == "file"
 
     def test_owner(self, path1, tmpdir):
-        from grp import getgrgid  # type:ignore[attr-defined]
-        from pwd import getpwuid  # type:ignore[attr-defined]
+        from grp import getgrgid  # type:ignore[attr-defined,unused-ignore]
+        from pwd import getpwuid  # type:ignore[attr-defined,unused-ignore]
 
         stat = path1.stat()
         assert stat.path == path1
@@ -1403,6 +1405,7 @@ class TestPOSIXLocalPath:
         import time
 
         path = tmpdir.ensure("samplefile")
+        # Do not use _pytest.timing here, as we do not want time mocking to affect this test.
         now = time.time()
         atime1 = path.atime()
         # we could wait here but timer resolution is very
